
#if !defined(NDEBUG)
#define _CRTDBG_MAP_ALLOC
#include <crtdbg.h>
#endif

#include <SAPS/SupportLib.h>
#include <assert.h>


EffectBase::EffectBase(PluginBase *plug, EffectInfoBase const *fi)
{
  plugin_ = plug;
  info_ = fi;
  host_ = plug->host_;
}

EffectBase::~EffectBase()
{
  for (std::vector<BusImpl *>::iterator ptr = busses_.begin(), end = busses_.end();
      ptr != end; ++ptr)
  {
    delete *ptr;
  }
}


AudioBus *EffectBase::AddBus(AudioBusConfig const &config, wchar_t const *name, void *userRef)
{
  busses_.push_back(new AudioBus(config, name, this, userRef));
  return static_cast<AudioBus *>(busses_.back());
}

MidiBus *EffectBase::AddBus(BusConfig const &config, wchar_t const *name, void *userRef)
{
  busses_.push_back(new MidiBus(config, name, this, userRef));
  return static_cast<MidiBus *>(busses_.back());
}

TimeBus *EffectBase::AddBus(TimeInput const &config, wchar_t const *name, void *userRef)
{
  busses_.push_back(new TimeBus(config, name, this, userRef));
  return static_cast<TimeBus *>(busses_.back());
}

FloatParameter *EffectBase::AddParameter(FloatRange const &range, wchar_t const *name, wchar_t const *unit, void *userRef)
{
  FloatParameter *fp = new FloatParameter(range, name, unit, this, userRef);
  parameters_.push_back(fp);
  return fp;
}




void EffectBase::DoEvents(size32 count, ItfEvent **events)
{
  assert(!"implement me");
}

void EffectBase::Reconfig(ItfConfig *config)
{
  blockSize_ = config->BlockSize();
  sampleRate_ = config->SampleRate();
  for (size_t i = 0; i != busses_.size(); ++i)
  {
    busses_[i]->Reconfig(config);
  }
}

size32 EffectBase::BusCount()
{
  return busses_.size();
}

ItfBus *EffectBase::Bus(size32 index)
{
  if (index >= busses_.size())
    return 0;
  return busses_[index]->Bus();
}

void EffectBase::Commit()
{
  assert(!"not implemented");
}

void EffectBase::Idle(size32 estMs)
{
  assert(!"not implemented");
}

ItfWindow *EffectBase::OpenGui()
{
  assert(!"not implemented");
  return 0;
}

size32 EffectBase::SamplesTail()
{
  assert(!"not implemented");
  return 0;
}

void EffectBase::EnterRealtime(TimePos const &atPos)
{
  assert(!"not implemented");
}

void EffectBase::Process()
{
  assert(!"not implemented");
}

void EffectBase::LeaveRealtime()
{
  assert(!"not implemented");
}

void EffectBase::Dispose()
{
  delete this;
}

ItfPlugin *EffectBase::Plugin()
{
  return plugin_;
}

void *EffectBase::Extension(wchar_t const *name)
{
  return 0;
}

